
#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include <winreg.h>

typedef int (__stdcall *CAMINIT)(DWORD dw1, DWORD *dw2);
typedef int (__stdcall *CAMDEINIT)();
typedef int (__stdcall *CAMFLASH)(DWORD dwValue);

// Helper
char *append_char ( const char *s, const char c )
{
  size_t len = strlen ( s );
  char *ret = new char[len + 2];

  strcpy ( ret, s );
  ret[len] = c;
  ret[len + 1] = '\0';

  return ret;
}

// Helper
void logger(const char *format, ...)
{
#ifdef DEBUG
    // Open file
    FILE *handle;
    handle = fopen("\\torchbutton.txt", "ab");

    // Append newline
    char* newformat = append_char(format, '\n');

    // Print to console
    va_list ap;
    va_start (ap, format);
    vprintf(newformat, ap);
    va_end (ap);

    // Call vfprintf to write with formatting
    va_start (ap, format);
    vfprintf(handle, newformat, ap);
    va_end (ap);

    // Close up
    fclose(handle);
#endif
}

// Helper
void DisplayError(wchar_t *error)
{
    MessageBox(NULL, error, L"Error", MB_OK);
}

int _tmain(int argc, _TCHAR* argv[])
{
    const wchar_t* EVENT_NAME = L"GLOBAL\\{3BC822CC-9330-4b94-983A-281418FA54AC}";
    const wchar_t* REGKEY_BASE = L"Software\\TorchButton";
    const wchar_t* REGVALUE_FLASHLIGHT_TIMEOUT = L"FlashlightTimeout";

    // Create a application-unique event, used to determine if the application is already running. If it
    // is, it is used to notify the currently running application.
    logger("Creating event...");
    HANDLE hEvent = CreateEvent(NULL, TRUE, FALSE, EVENT_NAME);
    if (!hEvent)
    {
        logger("Error creating event");
        DisplayError(L"Error creating event.");
        return 1;
    }

    // If we get the already exists error, it means the application is already running. In this case,
    // we simply set the event and get out of here.
    if (GetLastError() == ERROR_ALREADY_EXISTS)
    {
        logger("Application is already running, signalling the event so that it can stop");
        SetEvent(hEvent);
        CloseHandle(hEvent);
        return 0;
    }

    // Read the camera timeout from the registry.
    DWORD flashlightTimeout = 30;
    HKEY hKey = 0;

    // Open or create the key
    logger("Opening or creating (if nonexisting) the regkey...");
    if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, REGKEY_BASE, NULL, NULL, REG_OPTION_NON_VOLATILE, NULL, NULL, &hKey, NULL) == ERROR_SUCCESS)
    {
        // Read the value into flashlightTimeout
        logger("Reading the value of FlashlightTimeout...");
        DWORD dwSize = sizeof(DWORD);
        if (RegQueryValueEx(hKey, REGVALUE_FLASHLIGHT_TIMEOUT, NULL, NULL, (BYTE*)&flashlightTimeout, &dwSize) != ERROR_SUCCESS)
        {
            // An error occurred when reading the data, the value probably doesn't exist, so we will
            // set it with a default of 30 seconds.
            logger("An error occurred reading the timeout value, setting it with a default value...");
            flashlightTimeout = 30;
            DWORD dwType = REG_DWORD;
            if (RegSetValueEx(hKey, REGVALUE_FLASHLIGHT_TIMEOUT, NULL, dwType, (BYTE*)&flashlightTimeout, sizeof(DWORD)) != ERROR_SUCCESS)
            {
                // An error occurred when setting the default data
                logger("Error while setting the default value in HKLM\\Software\\TorchButton value 'FlashlightTimeout', aborting...");
                DisplayError(L"Error while setting the default value in HKLM\\Software\\TorchButton value 'FlashlightTimeout', aborting...");
                RegCloseKey(hKey);
                hKey = 0;
                return 1;
            }
        }
    }
    else
    {
        logger("Error opening registry key HKLM\\Software\\TorchButton, aborting...");
        DisplayError(L"Error opening registry key HKLM\\Software\\TorchButton, aborting...");
        return 1;
    }

    // Validate the flashlight timeout
    if (flashlightTimeout > 300)
    {
        logger("The configured timeout was larger than 300, setting it to 300...");
        flashlightTimeout = 300;
    }
    if (flashlightTimeout < 0)
    {
        logger("The configured timeout was less than 0, setting it to 0...");
        flashlightTimeout = 0;
    }

    // Load the camera module
    logger("Loading module...");
	HMODULE hCamera=LoadLibrary(L"HTCCamera1.dll");
	if (hCamera) {

        // Find and initialize the methods in the module
        logger("Finding and initializing module methods...");
		CAMINIT Camera_Init;
		CAMDEINIT Camera_Deinit;
		CAMFLASH Camera_FlashLight;
		Camera_Init=(CAMINIT)GetProcAddress(hCamera, L"Camera_Init");
		Camera_Deinit=(CAMDEINIT)GetProcAddress(hCamera, L"Camera_Deinit");
		Camera_FlashLight=(CAMFLASH)GetProcAddress(hCamera, L"Camera_FlashLight");

        // Only continue if we're successful
		if (Camera_Init && Camera_Deinit && Camera_FlashLight)
		{
            // Initialize the camera
            logger("Initializing the camera...");
			DWORD p1=0x00000001;
			DWORD p2=0x00000000;
			Camera_Init(p1, &p2);

            // Continue if initialized successfully
            if (p2 == 0)
            {
                // Turn on the light
                logger("Enabling the flashlight...");
                Camera_FlashLight(1);

                // Reset the event, just in case
                ResetEvent(hEvent);

                // We will wait for the event to be signalled, or for 30 seconds to pass to prevent burning the led.
                logger("Waiting for an event or timeout to pass...");
                WaitForSingleObject(hEvent, flashlightTimeout * 1000);

                // Turn off the light
                logger("Disabling the flashlight...");
                Camera_FlashLight(2);

                /*
    			// Find out which key was pressed
			    int pressed = -1;
			    for (int i=0; i < 255; i++)
			    {
				    if (GetAsyncKeyState(i))
				    {
					    pressed = i;
				    }
			    }

			    // If a key was (still) pressed, wait till it's no longer pressed
			    if (pressed != -1)
			    {
				    while (GetAsyncKeyState(pressed))
				    {
					    Sleep(100);
				    }
			    }*/
            }
            else
            {
                logger("Camera_Init reported an error while initializing.");
                DisplayError(L"Camera_Init reported an error while initializing. Aborting...");
            }

            // Deinitialize before quitting
            logger("Deinitializing the camera...");
			Camera_Deinit();
		}
		else
		{
            logger("Error locating the required methods in HTCCamera1.dll");
			DisplayError(L"Error locating the required methods in HTCCamera1.dll");
		}

        // Clean up
        logger("Cleaning up...");
		FreeLibrary(hCamera);
        CloseHandle(hEvent);
	}
	else 
	{
        logger("Error loading HTCCamera1.dll");
		DisplayError(L"Error loading HTCCamera1.dll");
	}
	return 0;
}
